/*
 *  UserInterface.java
 *
 *  Author : Ahmed Obied (ahmed.obied@gmail.com)
 *
 */

import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.custom.*;
import java.io.*;

public class UserInterface 
{
	private Display display;
	private Shell shell;
	private Text spamTrainPath, nspamTrainPath, inboxPath, testPath;
	private Button spamBrowse, nspamBrowse, startTraining, inboxBrowse, testBrowse, startClass, startTesting, singleCheck, doubleCheck;
	private Table trainingTable, nspamTable, spamTable;
	private ProgressBar spamBar, nspamBar, inboxBar, testingBar;
	private MenuItem menuItem, saveItem, quitItem;
	public Label spamTrainTotal, nspamTrainTotal, msgTotal, spamTotal, nspamTotal, spamClass, nspamClass, accuracy;
	private Label spamProcess, nspamProcess, inboxProcess;
	private boolean sTrain, nTrain;
	private Storage storage;
	
	public UserInterface()
	{
		storage = new Storage();
		display = new Display();
		shell = new Shell(display);
		shell.setText("Bayesian Spam Filtering - Ahmed Obied");
		shell.setLocation(0,0);
	}

	public void error(String title, String message)
	{
		MessageBox messageBox;
		
		messageBox = new MessageBox(shell,SWT.ICON_ERROR);
		messageBox.setText(title);
		messageBox.setMessage(message);
		messageBox.open();
	}
	
	public void incrementProgressBar(ProgressBar progressBar)
	{
		progressBar.setSelection(progressBar.getSelection() + 1);
	}
	
	public void loadPath()
	{
		File spamPath,nspamPath;
		final File[] spamFiles, nspamFiles;
		final int spamFilesTotal, nspamFilesTotal;
		
		nTrain = sTrain = true;
		spamPath = new File(spamTrainPath.getText());
		nspamPath = new File(nspamTrainPath.getText());
		spamFiles = spamPath.listFiles();
		nspamFiles = nspamPath.listFiles();
		spamFilesTotal = spamFiles.length;
		nspamFilesTotal = nspamFiles.length;
		spamTrainTotal.setText(new Integer(spamFilesTotal).toString());
		nspamTrainTotal.setText(new Integer(nspamFilesTotal).toString());
		spamBar.setMinimum(0);
		spamBar.setSelection(0);
		spamBar.setMaximum(spamFilesTotal);
		nspamBar.setMinimum(0);
		nspamBar.setSelection(0);
		nspamBar.setMaximum(nspamFilesTotal);
		startTraining.setEnabled(false);
		new Thread() {		
			public void run() {
				
				for (final int[] i = new int[1] ; i[0] < spamFilesTotal ; i[0]++) {
					FeatureExtractor featureExtractor;
					
					featureExtractor = new FeatureExtractor(spamFiles[i[0]]);
					featureExtractor.extractFeatures(storage,1);
					if (display.isDisposed() || shell.isDisposed()) 
							return;
					display.asyncExec(new Runnable() 
					{
						public void run() {
							spamProcess.setText(new Integer(spamBar.getSelection() + 1).toString());
							spamBar.setSelection(spamBar.getSelection() + 1);
						}
					});
				}
				sTrain = false;
				display.asyncExec(new Runnable() 
				{
					public void run() 
					{
						if (!sTrain && !nTrain)
							startTraining.setEnabled(true);
					}
				});
			}
		}.start();
		new Thread() {
			public void run() {
				for (final int[] i = new int[1] ; i[0] < nspamFilesTotal ; i[0]++) {
					FeatureExtractor featureExtractor;
					
					featureExtractor = new FeatureExtractor(nspamFiles[i[0]]);
					featureExtractor.extractFeatures(storage,2);
					if (display.isDisposed() || shell.isDisposed()) 
						return;
					display.asyncExec(new Runnable() 
					{
						public void run() 
						{
							nspamProcess.setText(new Integer(nspamBar.getSelection() + 1).toString());
							nspamBar.setSelection(nspamBar.getSelection() + 1);
						}
					});
				}
				nTrain = false;
				display.asyncExec(new Runnable() 
				{
					public void run() 
					{
						if (!nTrain && !sTrain) {
							startTraining.setEnabled(true);
							
						}
					}
				});
			}
		}.start();
	}
	
	public void loadInbox()
	{
		File inboxFilePath;
		final File[] inboxFiles;
		final int inboxFilesTotal;
		
		inboxFilePath = new File(inboxPath.getText());
		inboxFiles = inboxFilePath.listFiles();
		inboxFilesTotal = inboxFiles.length;
		msgTotal.setText(new Integer(inboxFilesTotal).toString());
		inboxBar.setMinimum(0);
		inboxBar.setSelection(0);
		inboxBar.setMaximum(inboxFilesTotal);
		startClass.setEnabled(false);
		new Thread() {		
			public void run() {
				
				for (final int[] i = new int[1] ; i[0] < inboxFilesTotal ; i[0]++) {
					FeatureExtractor featureExtractor;
					
					featureExtractor = new FeatureExtractor(inboxFiles[i[0]]);
					//featureExtractor.extractFeatures(database,1);
					if (display.isDisposed() || shell.isDisposed()) 
						return;
					display.asyncExec(new Runnable() 
					{
						public void run() {
							inboxProcess.setText(new Integer(inboxBar.getSelection() + 1).toString());
							inboxBar.setSelection(inboxBar.getSelection() + 1);
						}
					});
				}
				display.asyncExec(new Runnable() 
				{
					public void run() 
					{
						startClass.setEnabled(true);
					}
				});
			}
		}.start();
	}
	
	public void loadComponents()
	{
		Menu menu, subMenu;
		MenuItem mItem;
		GridLayout gridLayout;
		GridData gridData;
		ScrolledComposite trainScroll, nspamScroll, spamScroll;
		Group trainControls, trainGroup, spamGroup, nspamGroup, inboxGroup, testGroup;
		Label label;
		String[] trainTitles = {"Feature","Non-spam","Spam","Probability"};
		String[] emailTitles = {"Subject","Sender","Date","Classification"};
		
		gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		shell.setLayout(gridLayout);
		
		menu = new Menu(shell,SWT.BAR);
		shell.setMenuBar(menu);
		mItem = new MenuItem(menu,SWT.CASCADE);
		mItem.setText("File");
		subMenu = new Menu(shell,SWT.DROP_DOWN);
		mItem.setMenu(subMenu);
		menuItem = new MenuItem(subMenu,SWT.PUSH);
		menuItem.setText("Load Training File ...");
		menuItem.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{
				FileDialog file;
				String path;
				
				file = new FileDialog(shell,SWT.OPEN);
				path = file.open();
				if (path != null)
					System.out.println("Loading: ... " + path);
			}
		});
		
		saveItem = new MenuItem(subMenu,SWT.PUSH);
		saveItem.setText("Save Training Data As");
		saveItem.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{
				FileDialog file;
				String path;
				
				file = new FileDialog(shell,SWT.SAVE);
				path = file.open();
				if (path != null)
					System.out.println("Save to: " + path);
			}
		});
		
		mItem = new MenuItem(subMenu,SWT.SEPARATOR);
		quitItem = new MenuItem(subMenu,SWT.PUSH);
		quitItem.setText("Quit");
		quitItem.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{
				display.dispose();
			//	database.disconnect();
				System.exit(0);
			}
		});
		
		trainControls = new Group(shell,SWT.NONE);
		trainControls.setText("Training Controls");
		gridLayout = new GridLayout();
		gridLayout.numColumns = 4;
		gridData = new GridData(GridData.VERTICAL_ALIGN_FILL|GridData.HORIZONTAL_ALIGN_FILL);
		trainControls.setLayout(gridLayout);
		trainControls.setLayoutData(gridData);
		
		label = new Label(trainControls,SWT.NONE);
		label.setText("Path to the spam training set ");
		label = new Label(trainControls,SWT.NONE);
		label.setText(":");
		
		spamTrainPath = new Text(trainControls,SWT.SINGLE|SWT.BORDER);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 300;
		spamTrainPath.setLayoutData(gridData);
		
		spamBrowse = new Button(trainControls,SWT.PUSH);
		spamBrowse.setText("Browse ... ");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 100;
		spamBrowse.setLayoutData(gridData);
		spamBrowse.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{
				DirectoryDialog directory;
				String path;
				
				directory = new DirectoryDialog(shell);
				path = directory.open();
				if (path != null) {
					spamTrainPath.setText(path);
				}
			}
		});
		
		label = new Label(trainControls,SWT.NONE);
		label.setText("Path to the non-spam training set ");
		label = new Label(trainControls,SWT.NONE);
		label.setText(":");
		
		nspamTrainPath = new Text(trainControls,SWT.SINGLE|SWT.BORDER);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 300;
		nspamTrainPath.setLayoutData(gridData);
		
		nspamBrowse = new Button(trainControls,SWT.PUSH);
		nspamBrowse.setText("Browse ... ");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 100;
		nspamBrowse.setLayoutData(gridData);
		nspamBrowse.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{
				DirectoryDialog directory;
				String path;
				
				directory = new DirectoryDialog(shell);
				path = directory.open();
				if (path != null)
					nspamTrainPath.setText(path);
			}
		});
		
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalSpan = 4;
		gridData.widthHint = 110;
		startTraining = new Button(trainControls,SWT.PUSH);
		startTraining.setText("Start Training");
		startTraining.setLayoutData(gridData);
		startTraining.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{	
				if (nspamTrainPath.getText().equals("") || spamTrainPath.getText().equals(""))
					error("Alert","You need to specify the paths to both the spam and non-spam directories before you can start the training");
				else 
					loadPath();
			}
		});
		
		label = new Label(trainControls,SWT.NONE);
		label.setText("Total number of spam messages ");
		label = new Label(trainControls,SWT.NONE);
		label.setText(":");
		spamTrainTotal = new Label(trainControls,SWT.NONE);
		spamTrainTotal.setText("0");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.horizontalSpan = 2;
		spamTrainTotal.setLayoutData(gridData);
		
		label = new Label(trainControls,SWT.NONE);
		label.setText("Total number of non-spam messages ");
		label = new Label(trainControls,SWT.NONE);
		label.setText(":");
		nspamTrainTotal = new Label(trainControls,SWT.NONE);
		nspamTrainTotal.setText("0");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.horizontalSpan = 2;
		nspamTrainTotal.setLayoutData(gridData);
	
		label = new Label(trainControls,SWT.NONE);
		label.setText("Processing spam messages ");
		label = new Label(trainControls,SWT.NONE);
		label.setText(":");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		//gridData.horizontalSpan = 2;
		gridData.widthHint = 50;
		spamBar = new ProgressBar(trainControls,SWT.NONE);
		spamBar.setLayoutData(gridData);
		spamProcess = new Label(trainControls,SWT.NONE);
		spamProcess.setText("0");
		spamProcess.setLayoutData(gridData);
		
		label = new Label(trainControls,SWT.NONE);
		label.setText("Processing non-spam messages ");
		label = new Label(trainControls,SWT.NONE);
		label.setText(":");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		//gridData.horizontalSpan = 2;
		gridData.widthHint = 50;
		
		nspamBar = new ProgressBar(trainControls,SWT.NONE);
		nspamBar.setLayoutData(gridData);
		nspamProcess = new Label(trainControls,SWT.NONE);
		nspamProcess.setText("0");
		nspamProcess.setLayoutData(gridData);
		
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.horizontalSpan = 4;
		singleCheck = new Button(trainControls,SWT.CHECK);
		singleCheck.setText("Enable feature extraction based on single tokens");
		singleCheck.setLayoutData(gridData);
		singleCheck.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{
				System.out.println("single check");
			}
		});
		doubleCheck = new Button(trainControls,SWT.CHECK);
		doubleCheck.setText("Enable feature extraction based on double token chains");
		doubleCheck.setLayoutData(gridData);
		doubleCheck.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{
				System.out.println("double check");
			}
		});
		
		trainGroup = new Group(shell,SWT.NONE);
		trainGroup.setText("Training History");
		gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL|GridData.VERTICAL_ALIGN_FILL);	
		trainGroup.setLayout(gridLayout);
		trainGroup.setLayoutData(gridData);
		
		trainScroll = new ScrolledComposite(trainGroup,SWT.H_SCROLL|SWT.V_SCROLL);
		trainScroll.setExpandHorizontal(true);
		trainScroll.setExpandVertical(true);
		trainScroll.setMinWidth(595);
		trainScroll.setMinHeight(200);
		
		trainingTable = new Table(trainScroll,SWT.BORDER|SWT.FULL_SELECTION|SWT.V_SCROLL);		
		trainingTable.setLinesVisible(true);
		trainingTable.setHeaderVisible(true);
		trainingTable.setSize(612,217);
		for (int i = 0 ; i < trainTitles.length ; i++) {
			TableColumn column = new TableColumn(trainingTable,SWT.NONE);
			column.setText(trainTitles[i]);		
			if (i == 0)
				column.setWidth(280);
			else
				column.setWidth(100);
		}
	
		inboxGroup = new Group(shell,SWT.NONE);
		inboxGroup.setText("Inbox");
		gridLayout = new GridLayout();
		gridLayout.numColumns = 4;
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		inboxGroup.setLayout(gridLayout);
		inboxGroup.setLayoutData(gridData);
		
		label = new Label(inboxGroup,SWT.NONE);
		label.setText("Path to inbox file ");
		label = new Label(inboxGroup,SWT.NONE);
		label.setText(":");
			
		inboxPath = new Text(inboxGroup,SWT.SINGLE|SWT.BORDER);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 300;
		inboxPath.setLayoutData(gridData);
		
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.widthHint = 100;
		inboxBrowse = new Button(inboxGroup,SWT.PUSH);
		inboxBrowse.setText("Browse ...");
		inboxBrowse.setLayoutData(gridData);
		inboxBrowse.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{
				DirectoryDialog file;
				String path;
				
				file = new DirectoryDialog(shell,SWT.OPEN);
				path = file.open();
				if (path != null) {
					inboxPath.setText(path);
				}
			}
		});
		
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalSpan = 4;
		gridData.widthHint = 110;
		startClass = new Button(inboxGroup,SWT.PUSH);
		startClass.setText("Start Classification");
		startClass.setLayoutData(gridData);
		startClass.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{	
				if (inboxPath.getText().equals("")) {
					error("Alert","You need to specify the path to the inbox before you can start the classification.");
					return;
				}
				loadInbox();
			}
			
		});
		
		label = new Label(inboxGroup,SWT.NONE);
		label.setText("Total number of messages in inbox ");
		label = new Label(inboxGroup,SWT.NONE);
		label.setText(":");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.horizontalSpan = 2;
		msgTotal = new Label(inboxGroup,SWT.NONE);
		msgTotal.setText("0");
		msgTotal.setLayoutData(gridData);
		label = new Label(inboxGroup,SWT.NONE);
		label.setText("Total number of spam messages ");
		label = new Label(inboxGroup,SWT.NONE);
		label.setText(":");
		spamTotal = new Label(inboxGroup,SWT.NONE);
		spamTotal.setText("0");
		spamTotal.setLayoutData(gridData);
		label = new Label(inboxGroup,SWT.NONE);
		label.setText("Total number of non-spam messages ");
		label = new Label(inboxGroup,SWT.NONE);
		label.setText(":");
		nspamTotal = new Label(inboxGroup,SWT.NONE);
		nspamTotal.setText("0");
		nspamTotal.setLayoutData(gridData);
		label = new Label(inboxGroup,SWT.NONE);
		label.setText("Processing inbox ");
		label = new Label(inboxGroup,SWT.NONE);
		label.setText(":");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 50;
		inboxBar = new ProgressBar(inboxGroup,SWT.NONE);
		inboxBar.setLayoutData(gridData);
		inboxProcess = new Label(inboxGroup,SWT.NONE);
		inboxProcess.setText("0");
		inboxProcess.setLayoutData(gridData);
		
		testGroup = new Group(shell,SWT.NONE);
		testGroup.setText("Testing");
		gridLayout = new GridLayout();
		gridLayout.numColumns = 4;
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		testGroup.setLayout(gridLayout);
		testGroup.setLayoutData(gridData);
		
		label = new Label(testGroup,SWT.NONE);
		label.setText("Path to test file ");
		label = new Label(testGroup,SWT.NONE);
		label.setText(":");
		
		testPath = new Text(testGroup,SWT.SINGLE|SWT.BORDER);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 290;
		testPath.setLayoutData(gridData);
		
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.widthHint = 100;
		testBrowse = new Button(testGroup,SWT.PUSH);
		testBrowse.setText("Browse ...");
		testBrowse.setLayoutData(gridData);
		testBrowse.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{
				FileDialog file;
				String path;
				
				file = new FileDialog(shell,SWT.OPEN);
				path = file.open();
				if (path != null) {
					testPath.setText(path);
				}
			}
		});
		
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalSpan = 4;
		gridData.widthHint = 110;
		startTesting = new Button(testGroup,SWT.PUSH);
		startTesting.setText("Start Verfication");
		startTesting.setLayoutData(gridData);
		startTesting.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent(Event event)
			{
				if (testPath.getText().equals("")) 
					error("Alert","You need to specify the path to the test file before you can verify the classification.");
			}
		});
		
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		label = new Label(testGroup,SWT.NONE);
		label.setText("Non-spam messages classified as spam ");
		label = new Label(testGroup,SWT.NONE);
		label.setText(":");
		nspamClass = new Label(testGroup,SWT.NONE);
		nspamClass.setText("0");
		nspamClass.setLayoutData(gridData);
		label = new Label(testGroup,SWT.NONE);
		label.setText("Spam messages classified as non-spam ");
		label = new Label(testGroup,SWT.NONE);
		label.setText(":");
		spamClass = new Label(testGroup,SWT.NONE);
		spamClass.setText("0");
		spamClass.setLayoutData(gridData);
		label = new Label(testGroup,SWT.NONE);
		label.setText("Accuracy Rate");
		label = new Label(testGroup,SWT.NONE);
		label.setText(":");
		accuracy = new Label(testGroup,SWT.NONE);
		accuracy.setText("0");
		accuracy.setLayoutData(gridData);
		
		label = new Label(testGroup,SWT.NONE);
		label.setText("Verifiying classification ");
		label = new Label(testGroup,SWT.NONE);
		label.setText(":");
		gridLayout = new GridLayout();
		gridData.horizontalSpan = 2;
		gridData.widthHint = 410;
		testingBar = new ProgressBar(testGroup,SWT.NONE);
		testingBar.setLayoutData(gridData);
		
		nspamGroup = new Group(shell,SWT.NONE);
		nspamGroup.setText("Non-spam messages");
		gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL|GridData.VERTICAL_ALIGN_FILL);
		
		nspamGroup.setLayout(gridLayout);
		nspamGroup.setLayoutData(gridData);
		
		nspamScroll = new ScrolledComposite(nspamGroup,SWT.H_SCROLL|SWT.V_SCROLL);
		nspamScroll.setExpandHorizontal(true);
		nspamScroll.setExpandVertical(true);
		nspamScroll.setMinWidth(595);
		nspamScroll.setMinHeight(245);
		
		nspamTable = new Table(nspamScroll,SWT.BORDER|SWT.FULL_SELECTION|SWT.V_SCROLL);		
		nspamTable.setLinesVisible(true);
		nspamTable.setHeaderVisible(true);
		nspamTable.setSize(612,262);
		for (int i = 0 ; i < emailTitles.length ; i++) {
			TableColumn column = new TableColumn(nspamTable,SWT.NONE);
			column.setText(emailTitles[i]);		
			if (i == 0)
				column.setWidth(300);
			else
				column.setWidth(100);
		}
	
		spamGroup = new Group(shell,SWT.NONE);
		spamGroup.setText("Spam messages");
		gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL|GridData.VERTICAL_ALIGN_FILL);
		spamGroup.setLayout(gridLayout);
		spamGroup.setLayoutData(gridData);
		
		spamScroll = new ScrolledComposite(spamGroup,SWT.H_SCROLL|SWT.V_SCROLL);
		spamScroll.setExpandHorizontal(true);
		spamScroll.setExpandVertical(true);
		spamScroll.setMinWidth(595);
		spamScroll.setMinHeight(245);
		
		spamTable = new Table(spamScroll,SWT.BORDER|SWT.FULL_SELECTION|SWT.V_SCROLL);		
		spamTable.setLinesVisible(true);
		spamTable.setHeaderVisible(true);
		spamTable.setSize(612,262);
		for (int i = 0 ; i < emailTitles.length ; i++) {
			TableColumn column = new TableColumn(spamTable,SWT.NONE);
			column.setText(emailTitles[i]);		
			if (i == 0)
				column.setWidth(300);
			else
				column.setWidth(100);
		}
	}
	
	public void loadShell()
	{
		shell.pack();
		shell.open();
		while (!shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();
		display.dispose();
	}
}
